@@ -1,7 +1,10 @@
 package files_test
 
 import (
+	"fmt"
 	"os"
+	"path/filepath"
+	"runtime"
 	"strconv"
 	"strings"
 	"sync"
@@ -14,6 +17,8 @@ import (
 	"gopkg.in/yaml.v3"
 )
 
+var mtime = time.Date(2023, 11, 5, 23, 15, 17, 0, time.UTC)
+
 type testStruct struct {
 	Contents files.Contents `yaml:"contents"`
 }
@@ -38,12 +43,16 @@ contents:
 	require.Len(t, config.Contents, 2)
 	for _, f := range config.Contents {
 		t.Logf("%+#v\n", f)
-		require.Equal(t, f.Source, "a")
-		require.Equal(t, f.Destination, "b")
+		require.Equal(t, "a", f.Source)
+		require.Equal(t, "b", f.Destination)
 	}
 }
 
-func TestDeepPathsWithGlob(t *testing.T) {
+func TestDeepPathsWithGlobAndUmask(t *testing.T) {
+	path := filepath.Join(t.TempDir(), "foo", "bar", "zaz", "file.txt")
+	// create a bunch of files with bad permissions
+	require.NoError(t, os.MkdirAll(filepath.Dir(path), 0o777))
+	require.NoError(t, os.WriteFile(path, nil, 0o777))
 	var config testStruct
 	dec := yaml.NewDecoder(strings.NewReader(`---
 contents:
@@ -52,19 +61,37 @@ contents:
   file_info:
     mode: 0644
     mtime: 2008-01-02T15:04:05Z
+- src: testdata/deep-paths/
+  dst: /bar
+- src: ` + path + `
+  dst: /foo/file.txt
 `))
 	dec.KnownFields(true)
 	err := dec.Decode(&config)
 	require.NoError(t, err)
-	require.Len(t, config.Contents, 1)
-	parsedContents, err := files.PrepareForPackager(config.Contents, "", false)
+	require.Len(t, config.Contents, 3)
+	parsedContents, err := files.PrepareForPackager(
+		config.Contents,
+		0o133,
+		"",
+		false,
+		mtime,
+	)
 	require.NoError(t, err)
 	for _, c := range parsedContents {
 		switch c.Source {
 		case "testdata/globtest/nested/b.txt":
 			require.Equal(t, "/bla/nested/b.txt", c.Destination)
+			require.Equal(t, "-rw-r--r--", c.Mode().String())
 		case "testdata/globtest/multi-nested/subdir/c.txt":
 			require.Equal(t, "/bla/multi-nested/subdir/c.txt", c.Destination)
+			require.Equal(t, "-rw-r--r--", c.Mode().String())
+		case "testdata/deep-paths/nested1/nested2/a.txt":
+			require.Equal(t, "/bar/nested1/nested2/a.txt", c.Destination)
+			require.Equal(t, "-rw-r--r--", c.Mode().String())
+		case path:
+			require.Equal(t, "/foo/file.txt", c.Destination)
+			require.Equal(t, "-rw-r--r--", c.Mode().String())
 		}
 	}
 }
@@ -80,7 +107,13 @@ contents:
 	err := dec.Decode(&config)
 	require.NoError(t, err)
 	require.Len(t, config.Contents, 1)
-	parsedContents, err := files.PrepareForPackager(config.Contents, "", true)
+	parsedContents, err := files.PrepareForPackager(
+		config.Contents,
+		0,
+		"",
+		true,
+		mtime,
+	)
 	require.NoError(t, err)
 	present := false
 
@@ -110,7 +143,13 @@ contents:
 	err := dec.Decode(&config)
 	require.NoError(t, err)
 
-	config.Contents, err = files.PrepareForPackager(config.Contents, "", true)
+	config.Contents, err = files.PrepareForPackager(
+		config.Contents,
+		0,
+		"",
+		true,
+		mtime,
+	)
 	require.NoError(t, err)
 	require.Len(t, config.Contents, 1)
 
@@ -118,8 +157,8 @@ contents:
 	require.NoError(t, err)
 
 	f := config.Contents[0]
-	require.Equal(t, f.Source, "files_test.go")
-	require.Equal(t, f.Destination, "/b")
+	require.Equal(t, "files_test.go", f.Source)
+	require.Equal(t, "/b", f.Destination)
 	require.Equal(t, f.FileInfo.Mode, fi.Mode())
 	require.Equal(t, f.FileInfo.MTime, fi.ModTime())
 }
@@ -140,7 +179,13 @@ contents:
 	err := dec.Decode(&config)
 	require.NoError(t, err)
 
-	config.Contents, err = files.PrepareForPackager(config.Contents, "rpm", true)
+	config.Contents, err = files.PrepareForPackager(
+		config.Contents,
+		0,
+		"rpm",
+		true,
+		mtime,
+	)
 	require.NoError(t, err)
 	require.Len(t, config.Contents, 1)
 
@@ -148,8 +193,8 @@ contents:
 	require.NoError(t, err)
 
 	f := config.Contents[0]
-	require.Equal(t, f.Source, "files_test.go")
-	require.Equal(t, f.Destination, "/b")
+	require.Equal(t, "files_test.go", f.Source)
+	require.Equal(t, "/b", f.Destination)
 	require.Equal(t, f.FileInfo.Mode, os.FileMode(0o123))
 	require.Equal(t, f.FileInfo.MTime, ct)
 }
@@ -172,7 +217,13 @@ contents:
 	err := dec.Decode(&config)
 	require.NoError(t, err)
 
-	config.Contents, err = files.PrepareForPackager(config.Contents, "", true)
+	config.Contents, err = files.PrepareForPackager(
+		config.Contents,
+		0,
+		"",
+		true,
+		mtime,
+	)
 	require.NoError(t, err)
 	config.Contents = withoutFileInfo(config.Contents)
 
@@ -232,7 +283,13 @@ contents:
 		wg.Add(1)
 		go func() {
 			defer wg.Done()
-			_, err := files.PrepareForPackager(config.Contents, "", false)
+			_, err := files.PrepareForPackager(
+				config.Contents,
+				0,
+				"",
+				false,
+				mtime,
+			)
 			require.NoError(t, err)
 		}()
 	}
@@ -246,7 +303,13 @@ func TestCollision(t *testing.T) {
 			{Source: "../testdata/whatever2.conf", Destination: "/samedestination"},
 		}
 
-		_, err := files.PrepareForPackager(configuredFiles, "", true)
+		_, err := files.PrepareForPackager(
+			configuredFiles,
+			0,
+			"",
+			true,
+			mtime,
+		)
 		require.ErrorIs(t, err, files.ErrContentCollision)
 	})
 
@@ -256,7 +319,13 @@ func TestCollision(t *testing.T) {
 			{Source: "../testdata/whatever2.conf", Destination: "/samedestination", Packager: "bar"},
 		}
 
-		_, err := files.PrepareForPackager(configuredFiles, "foo", true)
+		_, err := files.PrepareForPackager(
+			configuredFiles,
+			0,
+			"foo",
+			true,
+			mtime,
+		)
 		require.NoError(t, err)
 	})
 
@@ -266,7 +335,13 @@ func TestCollision(t *testing.T) {
 			{Source: "../testdata/whatever2.conf", Destination: "/samedestination", Packager: ""},
 		}
 
-		_, err := files.PrepareForPackager(configuredFiles, "foo", true)
+		_, err := files.PrepareForPackager(
+			configuredFiles,
+			0,
+			"foo",
+			true,
+			mtime,
+		)
 		require.ErrorIs(t, err, files.ErrContentCollision)
 	})
 }
@@ -297,7 +372,13 @@ func TestDisableGlobbing(t *testing.T) {
 		content := testCase
 
 		t.Run(strconv.Itoa(i), func(t *testing.T) {
-			result, err := files.PrepareForPackager(files.Contents{&content}, "", disableGlobbing)
+			result, err := files.PrepareForPackager(
+				files.Contents{&content},
+				0,
+				"",
+				disableGlobbing,
+				mtime,
+			)
 			if err != nil {
 				t.Fatalf("expand content globs: %v", err)
 			}
@@ -336,12 +417,21 @@ func withoutImplicitDirs(contents files.Contents) files.Contents {
 }
 
 func TestGlobbingWhenFilesHaveBrackets(t *testing.T) {
-	result, err := files.PrepareForPackager(files.Contents{
-		{
-			Source:      "./testdata/\\{test\\}/",
-			Destination: ".",
+	if runtime.GOOS == "windows" {
+		t.Skip("doesn't work on windows")
+	}
+	result, err := files.PrepareForPackager(
+		files.Contents{
+			{
+				Source:      "./testdata/\\{test\\}/",
+				Destination: ".",
+			},
 		},
-	}, "", false)
+		0,
+		"",
+		false,
+		mtime,
+	)
 	if err != nil {
 		t.Fatalf("expand content globs: %v", err)
 	}
@@ -374,15 +464,21 @@ func TestGlobbingWhenFilesHaveBrackets(t *testing.T) {
 }
 
 func TestGlobbingFilesWithDifferentSizesWithFileInfo(t *testing.T) {
-	result, err := files.PrepareForPackager(files.Contents{
-		{
-			Source:      "./testdata/globtest/different-sizes/**/*",
-			Destination: ".",
-			FileInfo: &files.ContentFileInfo{
-				Mode: 0o777,
+	result, err := files.PrepareForPackager(
+		files.Contents{
+			{
+				Source:      "./testdata/globtest/different-sizes/**/*",
+				Destination: ".",
+				FileInfo: &files.ContentFileInfo{
+					Mode: 0o777,
+				},
 			},
 		},
-	}, "", false)
+		0,
+		"",
+		false,
+		mtime,
+	)
 	if err != nil {
 		t.Fatalf("expand content globs: %v", err)
 	}
@@ -399,12 +495,18 @@ func TestGlobbingFilesWithDifferentSizesWithFileInfo(t *testing.T) {
 }
 
 func TestDestEndsWithSlash(t *testing.T) {
-	result, err := files.PrepareForPackager(files.Contents{
-		{
-			Source:      "./testdata/globtest/a.txt",
-			Destination: "./foo/",
+	result, err := files.PrepareForPackager(
+		files.Contents{
+			{
+				Source:      "./testdata/globtest/a.txt",
+				Destination: "./foo/",
+			},
 		},
-	}, "", false)
+		0,
+		"",
+		false,
+		mtime,
+	)
 	result = withoutImplicitDirs(result)
 	require.NoError(t, err)
 	require.Len(t, result, 1)
@@ -421,7 +523,13 @@ contents:
 `))
 	dec.KnownFields(true)
 	require.NoError(t, dec.Decode(&config))
-	_, err := files.PrepareForPackager(config.Contents, "", false)
+	_, err := files.PrepareForPackager(
+		config.Contents,
+		0,
+		"",
+		false,
+		mtime,
+	)
 	require.EqualError(t, err, "invalid content type: filr")
 }
 
@@ -452,17 +560,29 @@ contents:
 `))
 	dec.KnownFields(true)
 	require.NoError(t, dec.Decode(&config))
-	_, err := files.PrepareForPackager(config.Contents, "", false)
+	_, err := files.PrepareForPackager(
+		config.Contents,
+		0,
+		"",
+		false,
+		mtime,
+	)
 	require.NoError(t, err)
 }
 
 func TestImplicitDirectories(t *testing.T) {
-	results, err := files.PrepareForPackager(files.Contents{
-		{
-			Source:      "./testdata/globtest/a.txt",
-			Destination: "./foo/bar/baz",
+	results, err := files.PrepareForPackager(
+		files.Contents{
+			{
+				Source:      "./testdata/globtest/a.txt",
+				Destination: "./foo/bar/baz",
+			},
 		},
-	}, "", false)
+		0,
+		"",
+		false,
+		mtime,
+	)
 	require.NoError(t, err)
 
 	expected := files.Contents{
@@ -483,7 +603,7 @@ func TestImplicitDirectories(t *testing.T) {
 		},
 	}
 
-	require.Equal(t, withoutFileInfo(results), expected)
+	require.Equal(t, expected, withoutFileInfo(results))
 }
 
 func TestRelevantFiles(t *testing.T) {
@@ -535,7 +655,13 @@ func TestRelevantFiles(t *testing.T) {
 	}
 
 	t.Run("deb", func(t *testing.T) {
-		results, err := files.PrepareForPackager(contents, "deb", false)
+		results, err := files.PrepareForPackager(
+			contents,
+			0,
+			"deb",
+			false,
+			mtime,
+		)
 		require.NoError(t, err)
 		require.Equal(t, files.Contents{
 			{
@@ -558,7 +684,13 @@ func TestRelevantFiles(t *testing.T) {
 	})
 
 	t.Run("rpm", func(t *testing.T) {
-		results, err := files.PrepareForPackager(contents, "rpm", false)
+		results, err := files.PrepareForPackager(
+			contents,
+			0,
+			"rpm",
+			false,
+			mtime,
+		)
 		require.NoError(t, err)
 		require.Equal(t, files.Contents{
 			{
@@ -601,7 +733,13 @@ func TestRelevantFiles(t *testing.T) {
 	})
 
 	t.Run("apk", func(t *testing.T) {
-		results, err := files.PrepareForPackager(contents, "apk", false)
+		results, err := files.PrepareForPackager(
+			contents,
+			0,
+			"apk",
+			false,
+			mtime,
+		)
 		require.NoError(t, err)
 		require.Equal(t, files.Contents{
 			{
@@ -613,14 +751,105 @@ func TestRelevantFiles(t *testing.T) {
 	})
 }
 
-func TestTree(t *testing.T) {
-	results, err := files.PrepareForPackager(files.Contents{
-		{
-			Source:      "testdata/tree",
-			Destination: "/base",
-			Type:        files.TypeTree,
+func TestTreeOwner(t *testing.T) {
+	results, err := files.PrepareForPackager(
+		files.Contents{
+			{
+				Source:      filepath.Join("testdata", "tree"),
+				Destination: "/usr/share/foo",
+				Type:        files.TypeTree,
+				FileInfo: &files.ContentFileInfo{
+					Owner: "goreleaser",
+					Group: "goreleaser",
+					MTime: mtime,
+				},
+			},
 		},
-	}, "", false)
+		0,
+		"",
+		false,
+		mtime,
+	)
+	require.NoError(t, err)
+
+	for _, f := range results {
+		if f.Destination == "/usr/" || f.Destination == "/usr/share/" {
+			require.Equal(t, "root", f.FileInfo.Owner, f.Destination)
+			require.Equal(t, "root", f.FileInfo.Group, f.Destination)
+			continue
+		}
+		require.Equal(t, "goreleaser", f.FileInfo.Owner, f.Destination)
+		require.Equal(t, "goreleaser", f.FileInfo.Group, f.Destination)
+	}
+
+	require.Equal(t, files.Contents{
+		{
+			Source:      "",
+			Destination: "/usr/",
+			Type:        files.TypeImplicitDir,
+		},
+		{
+			Source:      "",
+			Destination: "/usr/share/",
+			Type:        files.TypeImplicitDir,
+		},
+		{
+			Source:      "",
+			Destination: "/usr/share/foo/",
+			Type:        files.TypeDir,
+		},
+		{
+			Source:      "",
+			Destination: "/usr/share/foo/files/",
+			Type:        files.TypeDir,
+		},
+		{
+			Source:      filepath.Join("testdata", "tree", "files", "a"),
+			Destination: "/usr/share/foo/files/a",
+			Type:        files.TypeFile,
+		},
+		{
+			Source:      "",
+			Destination: "/usr/share/foo/files/b/",
+			Type:        files.TypeDir,
+		},
+		{
+			Source:      filepath.Join("testdata", "tree", "files", "b", "c"),
+			Destination: "/usr/share/foo/files/b/c",
+			Type:        files.TypeFile,
+		},
+		{
+			Source:      "",
+			Destination: "/usr/share/foo/symlinks/",
+			Type:        files.TypeDir,
+		},
+		{
+			Source:      "/etc/foo",
+			Destination: "/usr/share/foo/symlinks/link1",
+			Type:        files.TypeSymlink,
+		},
+		{
+			Source:      "../files/a",
+			Destination: "/usr/share/foo/symlinks/link2",
+			Type:        files.TypeSymlink,
+		},
+	}, withoutFileInfo(results))
+}
+
+func TestTree(t *testing.T) {
+	results, err := files.PrepareForPackager(
+		files.Contents{
+			{
+				Source:      filepath.Join("testdata", "tree"),
+				Destination: "/base",
+				Type:        files.TypeTree,
+			},
+		},
+		0,
+		"",
+		false,
+		mtime,
+	)
 	require.NoError(t, err)
 
 	require.Equal(t, files.Contents{
@@ -635,7 +864,7 @@ func TestTree(t *testing.T) {
 			Type:        files.TypeDir,
 		},
 		{
-			Source:      "testdata/tree/files/a",
+			Source:      filepath.Join("testdata", "tree", "files", "a"),
 			Destination: "/base/files/a",
 			Type:        files.TypeFile,
 		},
@@ -645,7 +874,7 @@ func TestTree(t *testing.T) {
 			Type:        files.TypeDir,
 		},
 		{
-			Source:      "testdata/tree/files/b/c",
+			Source:      filepath.Join("testdata", "tree", "files", "b", "c"),
 			Destination: "/base/files/b/c",
 			Type:        files.TypeFile,
 		},
@@ -680,11 +909,14 @@ func withoutFileInfo(contents files.Contents) files.Contents {
 }
 
 func TestAsRelativePath(t *testing.T) {
+	sep := fmt.Sprintf("%c", filepath.Separator)
 	testCases := map[string]string{
 		"/etc/foo/":         "etc/foo/",
 		"./etc/foo":         "etc/foo",
 		"./././foo/../bar/": "bar/",
-		"/":                 "",
+		sep:                 "",
+		sep + sep:           "",
+		sep + strings.Join([]string{"foo", "bar", "zazz"}, sep): "foo/bar/zazz",
 	}
 
 	for input, expected := range testCases {
@@ -693,11 +925,14 @@ func TestAsRelativePath(t *testing.T) {
 }
 
 func TestAsExplicitRelativePath(t *testing.T) {
+	sep := fmt.Sprintf("%c", filepath.Separator)
 	testCases := map[string]string{
 		"/etc/foo/":         "./etc/foo/",
 		"./etc/foo":         "./etc/foo",
 		"./././foo/../bar/": "./bar/",
-		"/":                 "./",
+		sep:                 "./",
+		sep:                 "./",
+		sep + strings.Join([]string{"foo", "bar", "zazz"}, sep): "./foo/bar/zazz",
 	}
 
 	for input, expected := range testCases {
