@@ -15,6 +15,7 @@ import (
 	"github.com/goreleaser/chglog"
 	"github.com/goreleaser/nfpm/v2"
 	"github.com/goreleaser/nfpm/v2/files"
+	"github.com/goreleaser/nfpm/v2/internal/modtime"
 	"github.com/goreleaser/nfpm/v2/internal/sign"
 )
 
@@ -69,13 +70,15 @@ var archToRPM = map[string]string{
 	// TODO: other arches
 }
 
-func ensureValidArch(info *nfpm.Info) *nfpm.Info {
+func setDefaults(info *nfpm.Info) *nfpm.Info {
 	if info.RPM.Arch != "" {
 		info.Arch = info.RPM.Arch
 	} else if arch, ok := archToRPM[info.Arch]; ok {
 		info.Arch = arch
 	}
 
+	info.Release = defaultTo(info.Release, "1")
+
 	return info
 }
 
@@ -83,15 +86,16 @@ func ensureValidArch(info *nfpm.Info) *nfpm.Info {
 // to the conventions for RPM packages. See:
 // http://ftp.rpm.org/max-rpm/ch-rpm-file-format.html
 func (*RPM) ConventionalFileName(info *nfpm.Info) string {
-	info = ensureValidArch(info)
-
-	version := formatVersion(info)
-	if info.Release != "" {
-		version += "-" + info.Release
-	}
+	info = setDefaults(info)
 
 	// name-version-release.architecture.rpm
-	return fmt.Sprintf("%s-%s.%s.rpm", info.Name, version, info.Arch)
+	return fmt.Sprintf(
+		"%s-%s-%s.%s.rpm",
+		info.Name,
+		formatVersion(info),
+		defaultTo(info.Release, "1"),
+		info.Arch,
+	)
 }
 
 // ConventionalExtension returns the file name conventionally used for RPM packages
@@ -105,7 +109,7 @@ func (*RPM) Package(info *nfpm.Info, w io.Writer) (err error) {
 		meta *rpmpack.RPMMetaData
 		rpm  *rpmpack.RPM
 	)
-	info = ensureValidArch(info)
+	info = setDefaults(info)
 
 	err = nfpm.PrepareForPackager(info, packagerName)
 	if err != nil {
@@ -126,6 +130,11 @@ func (*RPM) Package(info *nfpm.Info, w io.Writer) (err error) {
 			info.RPM.Signature.KeyID,
 		))
 	}
+	if signFn := info.RPM.Signature.SignFn; signFn != nil {
+		rpm.SetPGPSigner(func(data []byte) ([]byte, error) {
+			return signFn(bytes.NewReader(data))
+		})
+	}
 
 	if err = createFilesInsideRPM(info, rpm); err != nil {
 		return err
@@ -248,7 +257,7 @@ func buildRPMMeta(info *nfpm.Info) (*rpmpack.RPMMetaData, error) {
 		Suggests:    suggests,
 		Conflicts:   conflicts,
 		Compressor:  info.RPM.Compression,
-		BuildTime:   time.Now(),
+		BuildTime:   modtime.Get(info.MTime),
 		BuildHost:   hostname,
 	}, nil
 }
@@ -336,7 +345,9 @@ func addScriptFiles(info *nfpm.Info, rpm *rpmpack.RPM) error {
 	return nil
 }
 
+// TODO: pass mtime down in all content types
 func createFilesInsideRPM(info *nfpm.Info, rpm *rpmpack.RPM) (err error) {
+	mtime := modtime.Get(info.MTime)
 	for _, content := range info.Contents {
 		if content.Packager != "" && content.Packager != packagerName {
 			continue
@@ -364,7 +375,7 @@ func createFilesInsideRPM(info *nfpm.Info, rpm *rpmpack.RPM) (err error) {
 		case files.TypeSymlink:
 			file = asRPMSymlink(content)
 		case files.TypeDir:
-			file = asRPMDirectory(content)
+			file = asRPMDirectory(content, mtime)
 		case files.TypeImplicitDir:
 			// we don't need to add imlicit directories to RPMs
 			continue
@@ -385,11 +396,11 @@ func createFilesInsideRPM(info *nfpm.Info, rpm *rpmpack.RPM) (err error) {
 	return nil
 }
 
-func asRPMDirectory(content *files.Content) *rpmpack.RPMFile {
+func asRPMDirectory(content *files.Content, mtime time.Time) *rpmpack.RPMFile {
 	return &rpmpack.RPMFile{
 		Name:  content.Destination,
 		Mode:  uint(content.Mode()) | tagDirectory,
-		MTime: uint32(time.Now().Unix()),
+		MTime: uint32(mtime.Unix()),
 		Owner: content.FileInfo.Owner,
 		Group: content.FileInfo.Group,
 	}
